SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

Object subclass: #Cart
	instanceVariableNames: 'products validProducts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 6/18/2012 17:33'!
withValidProducts: aSetOfProducts

	^ Cart new initializeWithValidProducts: aSetOfProducts.! !

!Cart methodsFor: 'comparing' stamp: 'GrossodelSel 6/18/2012 17:30'!
= anotherObject

	(self class = anotherObject class) ifFalse: [ ^ false ].
	
	^ (products = anotherObject products) and: [ validProducts = anotherObject validProducts ].! !

!Cart methodsFor: 'adding' stamp: 'GrossodelSel 6/18/2012 17:32'!
add: aProduct quantity: aQuantity

	(aQuantity <= 0) ifTrue:
		[ self error: self invalidQuantityErrorMessage ].
	
	(validProducts includes: aProduct) ifFalse:
		[ self error: self invalidProductErrorMessage ].
	
	products add: aProduct withOccurrences: aQuantity.! !

!Cart methodsFor: 'enumerating' stamp: 'GrossodelSel 6/18/2012 17:27'!
do: aBlock

	products valuesAndCounts keysDo: aBlock.! !

!Cart methodsFor: 'initializing' stamp: 'GrossodelSel 6/18/2012 17:32'!
initializeWithValidProducts: aSetOfProducts

	products := Bag new.
	validProducts := aSetOfProducts.! !

!Cart methodsFor: 'error' stamp: 'GrossodelSel 6/18/2012 17:32'!
invalidProductErrorMessage

	^ 'the specified product is invalid'.! !

!Cart methodsFor: 'error' stamp: 'GrossodelSel 6/13/2012 16:04'!
invalidQuantityErrorMessage

	^ 'the specified quantity must be greater than zero'.! !

!Cart methodsFor: 'testing' stamp: 'GrossodelSel 6/18/2012 17:27'!
isEmpty

	^ products isEmpty.! !

!Cart methodsFor: 'accessing' stamp: 'GrossodelSel 6/18/2012 17:55'!
products

	^ Bag newFrom: products.! !

!Cart methodsFor: 'accessing' stamp: 'GrossodelSel 6/18/2012 17:33'!
quantityOf: aProduct 

	^ products occurrencesOf: aProduct.! !

!Cart methodsFor: 'accessing' stamp: 'GrossodelSel 6/18/2012 17:30'!
validProducts

	^ validProducts! !

Object subclass: #Cashier
	instanceVariableNames: 'creditCardUses catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier class methodsFor: 'error' stamp: 'GrossodelSel 6/15/2012 18:42'!
canNotCheckOutAnEmptyCartErrorMessage

	^ 'Can not check out an empty cart'.! !

!Cashier class methodsFor: 'error' stamp: 'GrossodelSel 6/15/2012 18:41'!
canNotCheckOutWithExpiredCreditCardErrorMessage

	^ 'Can not check out with expired credit card'.! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 6/18/2012 17:17'!
withCatalog: aCatalog

	^ Cashier new initializeWithCatalog: aCatalog.! !

!Cashier methodsFor: 'check-out' stamp: 'GrossodelSel 6/18/2012 17:24'!
checkOutCart: aCart withCreditCard: aCreditCard

	| amount |

	(aCart isEmpty)
		ifTrue: [ self error:  Cashier canNotCheckOutAnEmptyCartErrorMessage].
	
	(self isExpired: aCreditCard)
		ifTrue: [ self error:  Cashier canNotCheckOutWithExpiredCreditCardErrorMessage].
	
	amount := 0.
	aCart do: [ :anElementOfTheCart |  amount := amount + (self priceOf: anElementOfTheCart) ].
	
	self debit: amount fromCreditCard: aCreditCard.
	
	^ amount.! !

!Cashier methodsFor: 'accessing' stamp: 'GrossodelSel 6/15/2012 17:55'!
creditCardUses

	^ creditCardUses.! !

!Cashier methodsFor: 'merchant-processor' stamp: 'GrossodelSel 6/18/2012 17:20'!
debit: anAmount fromCreditCard: aCreditCard 

	"Debit from the card on the Merchant Processor"
	
	creditCardUses := creditCardUses + 1.
! !

!Cashier methodsFor: 'initializing' stamp: 'GrossodelSel 6/15/2012 17:53'!
initialize

	creditCardUses := 0.! !

!Cashier methodsFor: 'initializing' stamp: 'GrossodelSel 6/18/2012 17:16'!
initializeWithCatalog: aCatalog

	catalog := aCatalog.
	creditCardUses := 0.! !

!Cashier methodsFor: 'credit-card' stamp: 'GrossodelSel 6/15/2012 18:25'!
isExpired: aCreditCard 

	| today |
	
	today := Date today.

	^ aCreditCard isExpiredOn: (MonthOfYear onMonthNumber: today monthIndex of: today year).
	
! !

!Cashier methodsFor: 'check-out' stamp: 'GrossodelSel 6/18/2012 17:21'!
priceOf: aProduct
	
	^ catalog priceOf: aProduct.! !

Object subclass: #Catalog
	instanceVariableNames: 'products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Catalog methodsFor: 'adding' stamp: 'GrossodelSel 6/18/2012 17:10'!
add: aProduct priced: aPrice

	products at: aProduct put: aPrice.! !

!Catalog methodsFor: 'initializing' stamp: 'GrossodelSel 6/18/2012 17:11'!
initialize

	products := Dictionary new.! !

!Catalog methodsFor: 'testing' stamp: 'GrossodelSel 6/18/2012 17:38'!
isEmpty

	^ products isEmpty.! !

!Catalog methodsFor: 'accessing' stamp: 'GrossodelSel 6/18/2012 17:13'!
priceOf: aProduct

	^ products at: aProduct.! !

!Catalog methodsFor: 'accessing' stamp: 'GrossodelSel 6/18/2012 17:12'!
products

	^ products keys.! !

TestCase subclass: #CartTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!CartTest class methodsFor: 'cart' stamp: 'GrossodelSel 6/15/2012 16:38'!
newNonEmptyCart

	^ self newEmptyCart add: self bookIsbn1 quantity: 3.! !

!CartTest methodsFor: 'as yet unclassified' stamp: 'GrossodelSel 6/13/2012 16:15'!
aBookIsbnOutOfEditorial

	^ 'this book isbn is not in the Editorial'.! !

!CartTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:32'!
bookIsbn1
	
	^ 'bookIsbn1'.! !

!CartTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:48'!
bookIsbn2
	
	^ 'bookIsbn2'.! !

!CartTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:48'!
bookIsbn3
	
	^ 'bookIsbn3'.! !

!CartTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/13/2012 16:48'!
bookIsbnsOfEditorial

	^ Set with: self bookIsbn1 with: self bookIsbn2 with: self bookIsbn3.! !

!CartTest methodsFor: 'error' stamp: 'GrossodelSel 6/15/2012 16:18'!
canNotAddAnInvalidQuantityOfBookIsbnsErrorMessage

	^ 'the specified quantity must be greater than zero'.! !

!CartTest methodsFor: 'error' stamp: 'GrossodelSel 6/18/2012 17:34'!
canNotAddToCartABookIsbnThatIsNotFromTheEditorialErrorMessage
	
	^ 'the specified product is invalid'.! !

!CartTest methodsFor: 'cart' stamp: 'GrossodelSel 6/15/2012 16:38'!
newCartWithOneCopyOfEachBookIsbn

	| cart |
	cart := self newEmptyCart.
	
	self bookIsbnsOfEditorial do:
		[ :aBookIsbn | cart add: aBookIsbn quantity: 1 ].
	
	^ cart.! !

!CartTest methodsFor: 'cart' stamp: 'GrossodelSel 6/18/2012 17:33'!
newEmptyCart

	^ Cart withValidProducts: self bookIsbnsOfEditorial.! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/15/2012 16:38'!
test01CartShouldBeEmptyAfterCreation

	self assert: self newEmptyCart isEmpty.! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/15/2012 16:38'!
test02CartShouldNotBeEmptyAfterAddingABookIsbnOfTheEditorial
	
	| cart |
	cart := self newEmptyCart.
	
	cart add: (self bookIsbn1) quantity: 1.
	
	self deny: cart isEmpty.! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 18:10'!
test03CartProductsShouldRemainAfterAddingABookIsbnOfTheEditorial

	| cart |
	cart := self newEmptyCart.
	cart add: (self bookIsbn1) quantity: 3.
	cart add: (self bookIsbn2) quantity: 2.
	cart add: (self bookIsbn3) quantity: 1.
	
	self assert: (cart quantityOf: self bookIsbn1) = 3.
	self assert: (cart quantityOf: self bookIsbn2) = 2.
	self assert: (cart quantityOf: self bookIsbn3) = 1.
	
	self assert: cart products size = 6.! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 18:18'!
test04CartsCanBeComparedWithEquals

	| emptyCart nonEmptyCart |
	emptyCart := self newEmptyCart.
	nonEmptyCart := self newCartWithOneCopyOfEachBookIsbn.
	
	self assert: emptyCart = emptyCart.
	self assert: nonEmptyCart = nonEmptyCart.
	self assert: self newEmptyCart = self newEmptyCart.
	self assert: self newCartWithOneCopyOfEachBookIsbn = self newCartWithOneCopyOfEachBookIsbn.
	
	self deny: self newEmptyCart = self newCartWithOneCopyOfEachBookIsbn.
	self deny: self newCartWithOneCopyOfEachBookIsbn = self newEmptyCart.! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 17:48'!
test05CanNotAddAnInvalidQuantityOfBookIsbnsToCart
	
	| anEmptyCart aNonEmptyCart |
	
	"Emtpy cart test"
	anEmptyCart := self newEmptyCart.
	self should: [ anEmptyCart add: self bookIsbn2 quantity: -1 ]
		raise: Error
		withExceptionDo: [ : anError |
			self assert: (anError messageText = self canNotAddAnInvalidQuantityOfBookIsbnsErrorMessage).
			self assert: (anEmptyCart = self newEmptyCart) ].
	
	
	"Non empty cart test"
	aNonEmptyCart := self newCartWithOneCopyOfEachBookIsbn.
	
	self should: [ aNonEmptyCart add: self bookIsbn2 quantity: -1 ]
		raise: Error
		withExceptionDo: [ : anError |
			self assert: (anError messageText = self canNotAddAnInvalidQuantityOfBookIsbnsErrorMessage).
			self assert: aNonEmptyCart = self newCartWithOneCopyOfEachBookIsbn ].
! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 17:48'!
test06CanNotAddToCartABookIsbnThatIsNotFromTheEditorial
	
	| anEmptyCart aNonEmptyCart |
	
	"Emtpy cart test"
	anEmptyCart := self newEmptyCart.
	self should: [ anEmptyCart add: (self aBookIsbnOutOfEditorial) quantity: 1 ]
		raise: Error
		withExceptionDo: [ : anError |
			self assert: (anError messageText = self canNotAddToCartABookIsbnThatIsNotFromTheEditorialErrorMessage).
			self assert: (anEmptyCart = self newEmptyCart) ].
	
	
	"Non empty cart test"
	aNonEmptyCart := self newCartWithOneCopyOfEachBookIsbn.
	
	self should: [ aNonEmptyCart add: (self aBookIsbnOutOfEditorial) quantity: 1 ]
		raise: Error
		withExceptionDo: [ : anError |
			self assert: (anError messageText = self canNotAddToCartABookIsbnThatIsNotFromTheEditorialErrorMessage).
			self assert: (aNonEmptyCart = self newCartWithOneCopyOfEachBookIsbn) ].
! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 17:48'!
test07CartShouldContainTheSameAmountOfAGivenBookIsbnThatWasAdded
	
	| cart |
	cart := self newEmptyCart.
	
	cart add: self bookIsbn1 quantity: 2.
	self assert: (cart quantityOf: self bookIsbn1) = 2.
	
	cart add: self bookIsbn1 quantity: 3.
	self assert: (cart quantityOf: self bookIsbn1) = 5.
	
	self assert: (cart quantityOf: self bookIsbn2) = 0.! !

!CartTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 17:48'!
test08CartCanBeEnumeratedUsingDo

	| initialSetOfBookIsbns enumeratedSetOfBookIsbns |
	
	initialSetOfBookIsbns := self bookIsbnsOfEditorial.
	enumeratedSetOfBookIsbns := Set new.
	
	(self newCartWithOneCopyOfEachBookIsbn) do: [ :aBookIsbn |
		enumeratedSetOfBookIsbns add: aBookIsbn ].
	
	self assert: initialSetOfBookIsbns = enumeratedSetOfBookIsbns.
! !

TestCase subclass: #CashierTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!CashierTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/15/2012 18:14'!
bookIsbn1
	
	^ 'bookIsbn1'.! !

!CashierTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/15/2012 18:14'!
bookIsbn2
	
	^ 'bookIsbn2'.! !

!CashierTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/15/2012 18:14'!
bookIsbn3
	
	^ 'bookIsbn3'.! !

!CashierTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/15/2012 18:14'!
bookIsbnsOfEditorial

	^ Set with: self bookIsbn1 with: self bookIsbn2 with: self bookIsbn3.! !

!CashierTest methodsFor: 'error' stamp: 'GrossodelSel 6/15/2012 18:26'!
canNotCheckOutAnEmptyCartErrorMessage
	
	^ 'Can not check out an empty cart'.! !

!CashierTest methodsFor: 'error' stamp: 'GrossodelSel 6/15/2012 18:38'!
canNotCheckOutWithExpiredCreditCardErrorMessage

	^ 'Can not check out with expired credit card'.! !

!CashierTest methodsFor: 'credit-card' stamp: 'GrossodelSel 6/15/2012 18:09'!
juansExpiredCreditCard

	^ CreditCard ownedBy: 'Juan' identifiedBy: 42 expiresOn: (MonthOfYear onMonthNumber: 1 of: Date today year - 1).! !

!CashierTest methodsFor: 'credit-card' stamp: 'GrossodelSel 6/15/2012 18:09'!
juansValidCreditCard

	^ CreditCard ownedBy: 'Juan' identifiedBy: 42 expiresOn: (MonthOfYear onMonthNumber: 1 of: Date today year + 1).! !

!CashierTest methodsFor: 'cart' stamp: 'GrossodelSel 6/15/2012 18:13'!
newCartWithOneCopyOfEachBookIsbn

	| cart |
	cart := self newEmptyCart.
	
	self bookIsbnsOfEditorial do:
		[ :aBookIsbn | cart add: aBookIsbn quantity: 1 ].
	
	^ cart.! !

!CashierTest methodsFor: 'cashier' stamp: 'GrossodelSel 6/18/2012 17:21'!
newCashier

	^ Cashier withCatalog: self newCatalog.! !

!CashierTest methodsFor: 'editorial-books' stamp: 'GrossodelSel 6/18/2012 17:14'!
newCatalog

	| catalog |
	
	catalog := Catalog new.
	catalog add: self bookIsbn1 priced: 10.
	catalog add: self bookIsbn2 priced: 20.
	catalog add: self bookIsbn3 priced: 50.
	
	^ catalog.! !

!CashierTest methodsFor: 'cart' stamp: 'GrossodelSel 6/18/2012 17:33'!
newEmptyCart

	^ Cart withValidProducts: self bookIsbnsOfEditorial.! !

!CashierTest methodsFor: 'test' stamp: 'GrossodelSel 6/15/2012 18:10'!
test01CashierShouldHaveZeroCreditCardUsesWhenCreated

	self assert: self newCashier creditCardUses = 0.! !

!CashierTest methodsFor: 'test' stamp: 'GrossodelSel 6/15/2012 18:10'!
test02CashierShouldIncreaseCreditCardUsesOnSuccessfulCheckOut

	| cashier creditCardUses |
	
	cashier := self newCashier.
	creditCardUses := cashier creditCardUses.
	
	cashier checkOutCart: self newCartWithOneCopyOfEachBookIsbn withCreditCard: self juansValidCreditCard.
	self assert: cashier creditCardUses = (creditCardUses + 1).
	
	cashier checkOutCart: self newCartWithOneCopyOfEachBookIsbn withCreditCard: self juansValidCreditCard.
	self assert: cashier creditCardUses = (creditCardUses + 2).
! !

!CashierTest methodsFor: 'test' stamp: 'GrossodelSel 6/15/2012 18:43'!
test03CashierCanNotCheckOutAnEmptyCart

	| cashier creditCardUses |
	
	cashier := self newCashier.
	creditCardUses := cashier creditCardUses.
	
	self should: [ cashier checkOutCart: self newEmptyCart withCreditCard: self juansValidCreditCard ]
		raise: Error
		withExceptionDo: [ : anError |
			self assert: (anError messageText = self canNotCheckOutAnEmptyCartErrorMessage).
			self assert: cashier creditCardUses = creditCardUses ].! !

!CashierTest methodsFor: 'test' stamp: 'GrossodelSel 6/15/2012 18:38'!
test04CashierCanNotCheckOutWithExpiredCreditCard

	| cashier creditCardUses |
	
	cashier := self newCashier.
	creditCardUses := cashier creditCardUses.
	
	self should: [ cashier checkOutCart: self newCartWithOneCopyOfEachBookIsbn withCreditCard: self juansExpiredCreditCard ]
		raise: Error
		withExceptionDo: [ : anError |
			self assert: (anError messageText = self canNotCheckOutWithExpiredCreditCardErrorMessage).
			self assert: cashier creditCardUses = creditCardUses ].! !

!CashierTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 17:23'!
test05CashierShouldChargeTheCorrectAmountOfMoney

	| cashier catalog cart totalPrice |
	
	cashier := self newCashier.
	catalog := self newCatalog.
	cart := self newCartWithOneCopyOfEachBookIsbn.
	
	totalPrice := 0.
	cart do:[ :aProduct | totalPrice := totalPrice + (catalog priceOf: aProduct) ].
	
	self assert: (cashier checkOutCart: cart withCreditCard: self juansValidCreditCard) = totalPrice.! !

TestCase subclass: #CatalogTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!CatalogTest methodsFor: 'catalog' stamp: 'GrossodelSel 6/18/2012 17:40'!
newEmptyCatalog

	^ Catalog new.! !

!CatalogTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 18:05'!
test01CatalogShouldBeEmptyWhenCreated

	self assert: self newEmptyCatalog isEmpty.! !

!CatalogTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 18:05'!
test02CatalogShouldNotBeEmptyAfterAddingAnElement

	self deny: (self newEmptyCatalog add: 'product0' priced: 100) isEmpty.! !

!CatalogTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 18:06'!
test03CatalogSetsAProductsPriceWhenAdded

	| catalog product price |
	
	product := 'product0'.
	price := 100.
	catalog := self newEmptyCatalog.
	catalog add: product priced: price.
	
	self assert: (catalog priceOf: product) = price.! !

!CatalogTest methodsFor: 'test' stamp: 'GrossodelSel 6/18/2012 18:11'!
test04CatalogProductsShouldRemainAfterAddingANewOne

	| catalog product1 product2 product3 product1Price product2Price product3Price |
	
	product1 := 'product1'.
	product2 := 'product2'.
	product3 := 'product3'.
	product1Price := 10.
	product2Price := 20.
	product3Price := 50.
	
	catalog := self newEmptyCatalog.
	catalog add: product1 priced: product1Price.
	catalog add: product2 priced: product2Price.
	catalog add: product3 priced: product3Price.
	
	self assert: (catalog priceOf: product1) = product1Price.
	self assert: (catalog priceOf: product2) = product2Price.
	self assert: (catalog priceOf: product3) = product3Price.
	
	self assert: (catalog products size) = 3.! !
